import { usePostSetDareRegionMutation } from '@api/myDARE';
import { usePostSetNmRegionMutation } from '@api/myRegions';
import React, { createContext, useContext, useState, useCallback } from 'react';
import { DareRegion, NmRegion } from 'src/screens/InAppScreens/TravelsScreen/utils/types';
import { StoreType, storage } from 'src/storage';

const RegionContext = createContext<any>(null);

export const useRegion = () => useContext(RegionContext);

export const RegionProvider = ({ children }: { children: React.ReactNode }) => {
  const [userData, setUserData] = useState({});
  const [nmRegions, setNmRegions] = useState<NmRegion[]>([]);
  const [dareRegions, setDareRegions] = useState<DareRegion[] | null>([]);
  const token = storage.get('token', StoreType.STRING) as string;
  const { mutate: updateNM } = usePostSetNmRegionMutation();
  const { mutate: updateDARE } = usePostSetDareRegionMutation();

  const handleUpdateNM = useCallback(
    (region: number, first: number, last: number, visits: number, quality: number) => {
      const updatedNM = {
        ...userData,
        first_visit_year: first,
        last_visit_year: last,
        best_visit_quality: quality,
        no_of_visits: visits,
        visited: visits > 0 ? true : false
      };

      const updatedNMData = {
        token,
        region,
        first,
        last,
        visits,
        quality
      };

      updateNM(updatedNMData);
      updatedNM && setUserData(updatedNM);
    },
    [userData, token]
  );

  const handleUpdateDare = useCallback(
    (region: number, visits: 0 | 1) => {
      const updatedDARE = { ...userData, visited: visits > 0 ? true : false };

      const updatedDareData = {
        token,
        region,
        visits
      };

      updateDARE(updatedDareData);
      updatedDARE && setUserData(updatedDARE);
    },
    [userData, token]
  );

  const handleUpdateNMList = useCallback(
    (id: number, first: number, last: number, visits: number, quality: number) => {
      const updatedNM = nmRegions.map((item) => {
        if (item.id === id) {
          return {
            ...item,
            year: first,
            last,
            quality,
            visits
          };
        }
        return item;
      });

      const updatedNMData = {
        token,
        region: id,
        first,
        last,
        visits,
        quality
      };

      updateNM(updatedNMData);
      updatedNM && setNmRegions(updatedNM);
    },
    [nmRegions]
  );

  const handleUpdateDareList = useCallback(
    (region: number, visits: 0 | 1) => {
      const updatedDARE = dareRegions?.map((item) => {
        if (item.id === region) {
          return {
            ...item,
            visited: String(visits)
          };
        }

        return item;
      });

      const updatedDareData = {
        token,
        region,
        visits
      };

      updateDARE(updatedDareData);
      updatedDARE && setDareRegions(updatedDARE);
    },
    [dareRegions]
  );

  return (
    <RegionContext.Provider
      value={{
        userData,
        setUserData,
        setNmRegions,
        nmRegions,
        setDareRegions,
        dareRegions,
        handleUpdateNM,
        handleUpdateDare,
        handleUpdateNMList,
        handleUpdateDareList
      }}
    >
      {children}
    </RegionContext.Provider>
  );
};
